#ifndef FileInfoModel_h
#define FileInfoModel_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "BaseFileInfoModel.h"
#include "Counter.h"
#include "FileInfo.h"
#include "FileSelectionClipboard.h"

#include <QMimeData>

class FileIconProvider;

//! FileInfo model. Stores file information for display in lists
class FileInfoModel : public BaseFileInfoModel<FileInfo>, public Counter
{

    //! Qt meta object declaration
    Q_OBJECT

    public:

    //! constructor
    FileInfoModel( QObject*, BaseFileInfo::TypeFlags = FileInfo::None );

    //! destructor
    virtual ~FileInfoModel()
    {}

    //! set icon provider
    void setIconProvider( FileIconProvider* value )
    { iconProvider_ = value; }

    //! set clipboard
    void setClipboard( FileSelectionClipboard );

    //! number of columns
    enum { nExtraColumns = 3 };

    //! column type enumeration
    enum ColumnType
    {
        Location = nColumns,
        Status,
        Action
    };

    //!@name methods reimplemented from base class
    //@{

    // flags
    virtual Qt::ItemFlags flags( const QModelIndex& ) const;

    // return data for a given index
    virtual QVariant data( const QModelIndex&, int ) const;

    //! mime type
    virtual QStringList mimeTypes( void ) const;

    //! mime data
    virtual QMimeData* mimeData( const QModelIndexList& ) const;

    //! drop mine data
    virtual bool dropMimeData( const QMimeData*, Qt::DropAction, int, int, const QModelIndex& );

    //! header data
    virtual QVariant headerData( int, Qt::Orientation, int = Qt::DisplayRole ) const;

    //! number of columns for a given index
    virtual int columnCount( const QModelIndex& = QModelIndex() ) const
    { return nColumns + nExtraColumns; }

    //@}

    //! used to sort FileInfo objects
    class SortFTor: public BaseFileInfoModel<FileInfo>::SortFTor
    {

        public:

        //! constructor
        SortFTor( const int& type, Qt::SortOrder order = Qt::AscendingOrder ):
            BaseFileInfoModel<FileInfo>::SortFTor( type, order )
            {}

        //! destructor
        virtual ~SortFTor( void )
        {}

        //! prediction
        virtual bool operator() ( const FileInfo&, const FileInfo& ) const;

    };

    Q_SIGNALS:

    //! request transfer (from drag and drop)
    void transferRequested( const FileInfo&, const FileInfo::List& );

    protected:

    //! sort
    virtual void _sort( int, Qt::SortOrder = Qt::AscendingOrder );

    //! icon matching given model index
    virtual QIcon _icon( const QModelIndex& ) const;

    protected Q_SLOTS:

    //! transfer request
    void _requestTransfer( void )
    { emit transferRequested( transferData_.destination(), transferData_.files() ); }

    private:

    //! true if local
    bool _isLocal( void ) const
    { return !( type_ & FileInfo::Remote ); }

    //! true if remote
    bool _isRemote( void ) const
    { return type_ & FileInfo::Remote; }

    //! base type (bitwise or of FileInfo::Type
    /*! this is used to distinguish between local and remote list views */
    BaseFileInfo::TypeFlags type_;

    //! icon provider
    FileIconProvider* iconProvider_;

    //! clipboard
    FileSelectionClipboard clipboard_;

    //! data structure used for drag and drop
    class TransferData
    {
        public:

        //! constructor
        TransferData( void )
        {}

        //! destructor
        virtual ~TransferData( void )
        {}

        //! set
        void set( const FileInfo& destination, const FileInfo::List& files )
        {
            destination_ = destination;
            files_ = files;
        }

        //! destination
        const FileInfo& destination( void ) const
        { return destination_; }

        //! files
        const FileInfo::List& files( void ) const
        { return files_; }

        private:

        //! destination
        FileInfo destination_;

        //! files
        FileInfo::List files_;

    };

    //! transfer data
    TransferData transferData_;

};

#endif
