#ifndef FtpMacro_h
#define FtpMacro_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Counter.h"
#include "FileInfo.h"

#include <QHash>
#include <QList>

//! ftp macros
/*! macros work for both local and remote files */
class FtpMacro: public Counter
{

    public:

    //! map command ids and types
    using List = QList<FtpMacro>;

    //! Ftp macro types
    enum Type
    {
        None,
        Connect,
        CloseConnection,
        Add,
        Clear,
        Abort,
        CopyRecursive,
        CreateDirectory,
        Rename,
        Remove,
        Open,
        ShowDirectory,
        ListRecursive,
        SizeRecursive,
        CheckConflicts,
        CheckFilesExist,
        CheckFilesCopy,
        CheckFilesMoved,
        ClearSelection,
        Select
    };

    //! constructor
    FtpMacro( const Type& = None );

    //! destructor
    virtual ~FtpMacro( void )
    {}

    //! equal to operator
    virtual bool operator == ( const FtpMacro& other ) const
    { return id() == other.id(); }

    //! less than operator
    virtual bool operator < ( const FtpMacro& other ) const
    { return id() < other.id(); }


    //!@name accessors
    //@{

    //! id
    int id( void ) const
    { return id_; }

    //! type
    const Type& type( void ) const
    { return type_; }

    //! return command name
    QString name( void ) const
    { return name( type() ); }

    //! map Type to command name
    static QString name( const Type& type );

    // true if arguments are passed
    bool hasArguments( void ) const
    { return !arguments_.isEmpty(); }

    //! argument size
    int argumentCount( void ) const
    { return arguments_.size(); }

    //! true if argument is present
    bool hasArgument( int index ) const
    { return index < arguments_.size(); }

    //! get argument
    const FileInfo& argument( int index ) const
    { return arguments_[index]; }

    //! arguments
    const FileInfo::List& arguments( void ) const
    { return arguments_; }

    //@}

    //!@name modifiers
    //@{

    //! add argument
    FtpMacro& addArgument( const FileInfo& argument )
    {
        arguments_ << argument;
        return *this;
    }

    //! push arguments
    FtpMacro& operator << (const FileInfo& argument )
    { return addArgument( argument ); }

    //! add argument
    FtpMacro& addArgument( const QString& argument )
    {
        addArgument( FileInfo( argument ) );
        return *this;
    }

    //! push arguments
    FtpMacro& operator << (const QString& argument )
    { return addArgument( argument ); }

    //! clear arguments
    void clearArguments( void )
    { arguments_.clear(); }

    //@}

    private:

    //! static id counter
    static unsigned int& _counter( void );

    //! command names
    using NameMap = QHash< Type, QString >;

    //! command names
    static NameMap& _names( void );

    //! command names
    static void _initializeNames( void );

    //! command type
    Type type_;

    //! arguments
    FileInfo::List arguments_;

    //! unique id
    int id_;

};

#endif
