#ifndef MainWindow_h
#define MainWindow_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "BaseMainWindow.h"
#include "BusyWidget.h"
#include "ConnectionAttributes.h"
#include "Counter.h"
#include "Debug.h"
#include "FileInfo.h"
#include "FileSelectionClipboard.h"
#include "FileSelectionFrame.h"
#include "FileSystemWatcher.h"
#include "FtpMacro.h"
#include "FtpCommand.h"

class DockWidget;
class FileIconProvider;
class FtpCommandDialog;
class FtpInterface;
class PlacesWidget;
class LogDialog;
class ProgressStatusBar;

#include <QTimer>

//* application main window
class MainWindow: public BaseMainWindow, public Counter
{

    //* Qt meta object declaration
    Q_OBJECT

    public:

    //* constructor;
    MainWindow( QWidget* parent = 0 );

    //*@name modifiers
    //@{

    //* open connection dialog with specified host
    void connectToHost( QString host, int = 21, QString = "anonymous", QString password = QString(), bool = false );

    //@}

    //*@name accessors
    //@{

    //* local selection frame
    FileSelectionFrame& localSelectionFrame( void ) const
    { return *localSelectionFrame_; }

    //* remote selection frame
    FileSelectionFrame& remoteSelectionFrame( void ) const
    { return *remoteSelectionFrame_; }

    //* selection frames
    FileSelectionFrame& selectionFrame( int type ) const
    { return *((type & FileInfo::Remote) ? remoteSelectionFrame_:localSelectionFrame_); }

    //* list of frames
    QList<FileSelectionFrame*> selectionFrames( void ) const
    { return QList<FileSelectionFrame*>() << localSelectionFrame_ << remoteSelectionFrame_; }

    //* places
    DockWidget& placesPanel( void ) const
    { return *placesPanel_; }

    //* active frame
    FileSelectionFrame& activeFrame( void ) const;

    //* ftp interface
    FtpInterface& interface( void ) const
    { return *interface_; }

    //* log dialog
    FtpCommandDialog& commandDialog( void ) const
    { return *commandDialog_; }

    //* log frame
    LogDialog& logDialog( void ) const
    { return *logDialog_; }

    //@}

    //*@name actions
    //@{

    //* connect
    QAction& toggleConnectionAction( void ) const
    { return *toggleConnectionAction_; }

    // home directory
    QAction& homeDirectoryAction( void ) const
    { return *homeDirectoryAction_; }

    //* previous directory
    QToolButton& previousDirectoryButton( void ) const
    { return *previousDirectoryButton_; }

    //* next directory
    QToolButton& nextDirectoryButton( void ) const
    { return *nextDirectoryButton_; }

    // directory above
    QAction& parentDirectoryAction( void ) const
    { return *parentDirectoryAction_; }

    //* reload
    QAction& reloadAction( void ) const
    { return *reloadAction_; }

    //* abort
    QAction& abortAction( void ) const
    { return *abortAction_; }

    //* toggle command frame
    QAction& showCommandDialogAction( void ) const
    { return *showCommandDialogAction_; }

    //* toggle log visibility
    QAction& showLogDialogAction( void ) const
    { return *showLogDialogAction_; }

    //* hidden files
    QAction& showHiddenFilesAction( void ) const
    { return *showHiddenFilesAction_; }

    //* icons view action
    QAction& iconsViewAction( void ) const
    { return *iconsViewAction_; }

    //* details view action
    QAction& detailsViewAction( void ) const
    { return *detailsViewAction_; }

    //@}

    Q_SIGNALS:

    // request macro
    void macroRequested( const FtpMacro& );

    // request macros
    void macrosRequested( const FtpMacro::List& );

    public Q_SLOTS:

    //* busy
    virtual void busy( void )
    { busyWidget_->start(); }

    //* busy
    virtual void idle( void )
    { busyWidget_->stop(); }

    protected:

    //* close window event handler
    void closeEvent( QCloseEvent* );

    //* check that transfer is possible
    bool _checkTransfer( const FileInfo&, const FileInfo::List& );

    //* paste selected files to destination
    void _paste( const FileInfo&, const FileInfo::List& );

    //* move selected files to destination
    /*! return updated clipboard */
    FileSelectionClipboard _move( const FileInfo&, const FileInfo::List& );

    //* get list of destination files by replacing source path by destination
    FileInfo::List _destinationList( const FileInfo&, const FileInfo::List&) const;

    //* update path
    void _updatePath( const FileInfo& );

    //* update window title
    void _updateWindowTitle( void );

    //* disable remote places
    void _disableRemotePlaces( void );

    //* request remote places update
    void _requestRemotePlacesUpdate( void )
    { _requestPlacesUpdate( FileInfo::Remote ); }

    //* request local places update
    void _requestLocalPlacesUpdate( void )
    { _requestPlacesUpdate( 0 ); }

    //* request places update
    void _requestPlacesUpdate( int );

    protected Q_SLOTS:

    //* update visibility actions
    void _updateVisibilityActions( void );

    //* navigation actions
    void _updateNavigationActions( void );

    //* view mode actions
    void _updateViewModeActions( void );

    //* update active frame
    void _updateActiveFrame( QWidget*, QWidget* );

    //* update view mode
    void _updateViewMode( FileSelectionFrame::ViewMode );

    //* update statusbar based on content
    void _updateStatusBarDelayed( void );

    //* update statusbar based on content
    void _updateStatusBar( void );

    //* toggle icons view mode
    void _toggleIconsViewMode( bool );

    //* toggle details view mode
    void _toggleDetailsViewMode( bool );

    //* show/hide hidden files
    void _toggleShowHiddenFiles( bool );

    //* connect/disconnect
    void _toggleConnection( void );

    //* abort transfer
    void _abort( void );

    //* file selected in one of the selection frame
    void _fileSelected( const FileInfo& );

    //* file selected in one of the selection frame
    void _placeSelected( const BaseFileInfo& );

    //* reload current directory
    void _reload( void );

    //* select previous directory from history
    void _previousDirectory( void );

    //* select next directory from history
    void _nextDirectory( void );

    //* select parent directory
    void _parentDirectory( void );

    //* select home directory
    void _homeDirectory( void );

    //* update clipboard
    void _updateClipboard( const FileSelectionClipboard& = FileSelectionClipboard() );

    //* paste (using drag and drop)
    void _pasteSelection( const FileInfo&, const FileInfo::List& );

    //* paste clipboard to path in argument
    void _pasteClipboard( const FileInfo& );

    //* open files
    void _openFiles( const FileInfo::List& );

    //* macro started
    void _macroStarted( const FtpMacro& );

    //* macro completed
    void _macroCompleted( const FtpMacro&, bool );

    //* command completed
    void _commandCompleted( const FtpCommand&, bool );

    //* update directory content
    void _updateContent( const FileInfo&, const FileInfo::List& );

    //* file info updated
    void _updateFileInfo( const FileInfo::List& files )
    {
        foreach( const FileInfo& fileInfo, files )
        { _updateFileInfo( fileInfo ); }
    }

    //* file info updated
    void _updateFileInfo( const FileInfo& );

    //* request places update
    void _requestPlacesUpdate( const BaseFileInfo& );

    //* connection state changed
    void _stateChanged( int );

    //* update buttons
    void _updateButtons( void );

    //* update buttons
    void _updateBusyButtons( bool );

    //* toggle command frame visibility
    void _showCommandDialog( void );

    //* command dialog hidden
    void _commandDialogHidden( void );

    //* toggle log visibility
    void _showLogDialog( void );

    //* log dialog hidden
    void _logDialogHidden( void );

    //* toggle show navigator
    void _toggleShowNavigator( bool );

    //* show file information
    void _fileInformation( const FileInfo&, const QIcon& );

    private Q_SLOTS:

    //* configuration
    void _updateConfiguration( void );

    private:

    //* install actions
    void _installActions( void );

    //* icon provider
    FileIconProvider* iconProvider_ = nullptr;

    //* local files
    FileSelectionFrame* localSelectionFrame_ = nullptr;

    //* remote files
    FileSelectionFrame* remoteSelectionFrame_ = nullptr;

    //* clipboard (for copy/paste actions)
    FileSelectionClipboard clipboard_;

    //* file system watcher, for local selection frame
    FileSystemWatcher fileSystemWatcher_;

    //* status bar
    ProgressStatusBar* statusbar_ = nullptr;

    //* statusbar update timer
    QTimer* statusbarUpdateTimer_ = nullptr;

    //* command list
    FtpCommandDialog* commandDialog_ = nullptr;

    //* log dialog
    LogDialog* logDialog_ = nullptr;

    //* Ftp interface
    FtpInterface* interface_ = nullptr;

    //* busy widget
    BusyWidget* busyWidget_ = nullptr;

    //* places
    PlacesWidget* placesWidget_ = nullptr;

    //* dock widget
    DockWidget* placesPanel_ = nullptr;

    //* current connection attributes
    ConnectionAttributes attributes_;

    //*@name actions
    //@{

    //* connection
    QAction* toggleConnectionAction_ = nullptr;

    //* home directory action
    QAction* homeDirectoryAction_ = nullptr;

    //* previous directory button
    QToolButton* previousDirectoryButton_ = nullptr;

    //* next directory button
    QToolButton* nextDirectoryButton_ = nullptr;

    //* parent directory action
    QAction* parentDirectoryAction_ = nullptr;

    //* reload
    QAction* reloadAction_ = nullptr;

    //* abort
    QAction* abortAction_ = nullptr;

    //* static menu actions
    QList<QAction*> menuActions_;

    //* toggle command frame action
    QAction* showCommandDialogAction_ = nullptr;

    //* toggle log action
    QAction* showLogDialogAction_ = nullptr;

    //* hidden files action
    QAction* showHiddenFilesAction_ = nullptr;

    //* icon view action
    QAction* iconsViewAction_ = nullptr;

    //* list view action
    QAction* detailsViewAction_ = nullptr;

    //@}

    //* show navigator
    bool showNavigator_ = true;

    //* true if enabled
    bool enabled_ = true;

};

#endif
